Ddoc

$(SPEC_S Lexical,

	In D, the lexical analysis is independent of the syntax parsing and the
	semantic analysis. The lexical analyzer splits the source text up into
	tokens. The lexical grammar describes what those tokens are. The D
	lexical grammar is designed to be suitable for high speed scanning, it
	has a minimum of special case rules, there is only one phase of
	translation, and to make it easy to write a correct scanner
	for. The tokens are readily recognizable by those familiar with C and
	C++.

<h3>Phases of Compilation</h3>

	The process of compiling is divided into multiple phases. Each phase
	has no dependence on subsequent phases. For example, the scanner is
	not perturbed by the semantic analyzer. This separation of the passes
	makes language tools like syntax 
	directed editors relatively easy to produce.
	It also is possible to compress D source by storing it in
	'tokenized' form.

$(OL
	<li> $(B source character set)<br>

	The source file is checked to see what character set it is,
	and the appropriate scanner is loaded. ASCII and UTF
	formats are accepted.

	$(LI $(B script line) $(BR)

	If the first line starts with $(GREEN #!) then the first line
	is ignored.)

	<li> $(B lexical analysis)<br>

	The source file is divided up into a sequence of tokens.
	$(LINK2 #specialtokens, Special tokens) are replaced with other tokens.
	$(LINK2 #specialtokenseq, Special token sequences)
	are processed and removed.

	<li> $(B syntax analysis)<br>

	The sequence of tokens is parsed to form syntax trees.

	<li> $(B semantic analysis)<br>

	The syntax trees are traversed to declare variables, load symbol tables, assign 
	types, and in general determine the meaning of the program.

	<li> $(B optimization)<br>

	Optimization is an optional pass that tries to rewrite the program
	in a semantically equivalent, but faster executing, version.

	<li> $(B code generation)<br>

	Instructions are selected from the target architecture to implement
	the semantics of the program. The typical result will be
	an object file, suitable for input to a linker.
)


<h3>Source Text</h3>

	D source text can be in one of the following formats:

	$(UL 
	<li> ASCII
	<li> UTF-8
	<li> UTF-16BE
	<li> UTF-16LE
	<li> UTF-32BE
	<li> UTF-32LE
	)

	UTF-8 is a superset of traditional 7-bit ASCII.
	One of the
	following UTF BOMs (Byte Order Marks) can be present at the beginning
	of the source text:
	<p>

	$(TABLE1
	<tr>
	<th> Format
	<th> BOM
	<tr>
	<td> UTF-8
	<td> EF BB BF
	<tr>
	<td> UTF-16BE
	<td> FE FF
	<tr>
	<td> UTF-16LE
	<td> FF FE
	<tr>
	<td> UTF-32BE
	<td> 00 00 FE FF
	<tr>
	<td> UTF-32LE
	<td> FF FE 00 00
	<tr>
	<td> ASCII
	<td> no BOM
	)

	$(P If the source file does not start with a BOM, then the first
	character must be less than or equal to U0000007F.)

	$(P There are no digraphs or trigraphs in D.)

	$(P The source text is decoded from its source representation
	into Unicode $(I Character)s.
	The $(I Character)s are further divided into:

	$(LINK2 #whitespace, white space),
	$(LINK2 #endofline, end of lines),
	$(LINK2 #comment, comments),
	$(LINK2 #specialtokens, special token sequences),
	$(LINK2 #tokens, tokens),
	all followed by $(LINK2 #eof, end of file).
	)

	$(P The source text is split into tokens using the maximal munch
	technique, i.e., the
	lexical analyzer tries to make the longest token it can. For example
	<code>>></code> is a right shift token,
	not two greater than tokens.
	)

<h3>$(LNAME2 eof, End of File)</h3>

$(GRAMMAR
$(I EndOfFile):
	$(I physical end of the file)
	\u0000
	\u001A
)

	The source text is terminated by whichever comes first.

<h3>$(LNAME2 endofline, End of Line)</h3>

$(GRAMMAR
$(I EndOfLine):
	\u000D
	\u000A
	\u000D \u000A
	$(I EndOfFile)
)

	There is no backslash line splicing, nor are there any limits
	on the length of a line.

<h3>$(LNAME2 whitespace, White Space)</h3>

$(GRAMMAR
$(I WhiteSpace):
	$(I Space)
	$(I Space) $(I WhiteSpace)

$(I Space):
	\u0020
	\u0009
	\u000B
	\u000C
)


<h3>$(LNAME2 comment, Comments)</h3>

$(GRAMMAR
$(I Comment):
	$(B /*) $(I Characters) $(B */)
	$(B //) $(I Characters) $(I EndOfLine)
	$(I NestingBlockComment)

$(I Characters):
	$(I Character)
	$(I Character) $(I Characters)

$(I NestingBlockComment):
	$(B /+) $(I NestingBlockCommentCharacters) $(B +/)

$(I NestingBlockCommentCharacters):
	$(I NestingBlockCommentCharacter)
	$(I NestingBlockCommentCharacter) $(I NestingBlockCommentCharacters)

$(I NestingBlockCommentCharacter):
	$(I Character)
	$(I NestingBlockComment)
)

	D has three kinds of comments:
	$(OL 
	<li> Block comments can span multiple lines, but do not nest.
	<li> Line comments terminate at the end of the line.
	<li> Nesting comments can span multiple lines and can nest.
	)

	$(P
	The contents of strings and comments are not tokenized.  Consequently,
	comment openings occurring within a string do not begin a comment, and
	string delimiters within a comment do not affect the recognition of
	comment closings and nested "/+" comment openings.  With the exception
	of "/+" occurring within a "/+" comment, comment openings within a
	comment are ignored.
	)

-------------
a = /+ // +/ 1;		// parses as if 'a = 1;'
a = /+ "+/" +/ 1";	// parses as if 'a = " +/ 1";'
a = /+ /* +/ */ 3;	// parses as if 'a = */ 3;'
-------------

	Comments cannot be used as token concatenators, for example,
	<code>abc/**/def</code> is two tokens, $(TT abc) and $(TT def),
	not one $(TT abcdef) token.

<h3>$(LNAME2 tokens, Tokens)</h3>

$(GRAMMAR
$(I Token):
	$(LINK2 #identifier, $(I Identifier))
	$(LINK2 #StringLiteral, $(I StringLiteral))
	$(LINK2 #characterliteral, $(I CharacterLiteral))
	$(LINK2 #integerliteral, $(I IntegerLiteral))
	$(LINK2 #floatliteral, $(I FloatLiteral))
	$(LINK2 #keyword, $(I Keyword))
	$(B /)
	$(B /=)
	$(B .)
	$(B ..)
	$(B ...)
	$(B &)
	$(B &=)
	$(B &&)
	$(B |)
	$(B |=)
	$(B ||)
	$(B -)
	$(B -=)
	$(B --)
	$(B +)
	$(B +=)
	$(B ++)
	$(B &lt;)
	$(B &lt;=)
	$(B &lt;&lt;)
	$(B &lt;&lt;=)
	$(B &lt;&gt;)
	$(B &lt;&gt=)
	$(B &gt;)
	$(B &gt;=)
	$(B &gt;&gt;=)
	$(B &gt;&gt;&gt;=)
	$(B &gt;&gt;)
	$(B &gt;&gt;&gt;)
	$(B !)
	$(B !=)
	$(B !==)
	$(B !&lt;&gt;)
	$(B !&lt;&gt;=)
	$(B !&lt;)
	$(B !&lt;=)
	$(B !&gt;)
	$(B !&gt;=)
	$(B !~)
	$(B $(LPAREN))
	$(B $(RPAREN))
	$(B [)
	$(B ])
	$(B {)
	$(B })
	$(B ?)
	$(B ,)
	$(B ;)
	$(B :)
	$(B $)
	$(B =)
	$(B ==)
	$(B ===)
	$(B *)
	$(B *=)
	$(B %)
	$(B %=)
	$(B ^)
	$(B ^=)
	$(B ~)
	$(B ~=)
	$(B ~~)
)

<h3>$(LNAME2 identifier, Identifiers)</h3>

$(GRAMMAR
$(I Identifier):
	$(I IdentiferStart)
	$(I IdentiferStart) $(I IdentifierChars)

$(I IdentifierChars):
	$(I IdentiferChar)
	$(I IdentiferChar) $(I IdentifierChars)

$(I IdentifierStart):
	$(B _)
	$(I Letter)
	$(I UniversalAlpha)

$(I IdentifierChar):
	$(I IdentiferStart)
	$(B 0)
	$(I NonZeroDigit)
)


	Identifiers start with a letter, $(B _), or universal alpha,
	and are followed by any number
	of letters, $(B _), digits, or universal alphas.
	Universal alphas are as defined in ISO/IEC 9899:1999(E) Appendix D.
	(This is the C99 Standard.)
	Identifiers can be arbitrarily long, and are case sensitive.
	Identifiers starting with $(B __) (two underscores) are reserved.

<h3>$(LNAME2 StringLiteral, String Literals)</h3>

$(GRAMMAR
$(I StringLiteral):
	$(I WysiwygString)
	$(I AlternateWysiwygString)
	$(I DoubleQuotedString)
	$(I EscapeSequence)
	$(I HexString)

$(I WysiwygString):
	$(B r") $(I WysiwygCharacters) $(B ") $(I Postfix<sub>opt</sub>)

$(I AlternateWysiwygString):
	$(B `) $(I WysiwygCharacters) $(B `) $(I Postfix<sub>opt</sub>)

$(I WysiwygCharacters):
	$(I WysiwygCharacter)
	$(I WysiwygCharacter) $(I WysiwygCharacters)

$(I WysiwygCharacter):
	$(I Character)
	$(I EndOfLine)

$(I DoubleQuotedString):
	$(B ") $(I DoubleQuotedCharacters) $(B ") $(I Postfix<sub>opt</sub>)

$(I DoubleQuotedCharacters):
	$(I DoubleQuotedCharacter)
	$(I DoubleQuotedCharacter) $(I DoubleQuotedCharacters)

$(I DoubleQuotedCharacter):
	$(I Character)
	$(I EscapeSequence)
	$(I EndOfLine)

$(LNAME2 EscapeSequence, $(I EscapeSequence)):
	$(B \')
	$(B \")
	$(B \?)
	$(B \\)
	$(B \a)
	$(B \b)
	$(B \f)
	$(B \n)
	$(B \r)
	$(B \t)
	$(B \v)
	$(B \) $(I EndOfFile)
	$(B \x) $(I HexDigit) $(I HexDigit)
	$(B \) $(I OctalDigit)
	$(B \) $(I OctalDigit) $(I OctalDigit)
	$(B \) $(I OctalDigit) $(I OctalDigit) $(I OctalDigit)
	$(B \u) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit)
	$(B \U) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit) $(I HexDigit)
	$(B \&amp;) $(LINK2 entity.html, $(I NamedCharacterEntity)) $(B ;)

$(I HexString):
	$(B x") $(I HexStringChars) $(B ") $(I Postfix<sub>opt</sub>)

$(I HexStringChars):
	$(I HexStringChar)
	$(I HexStringChar) $(I HexStringChars)

$(I HexStringChar)
	$(I HexDigit)
	$(I WhiteSpace)
	$(I EndOfLine)

$(I Postfix)
	$(B c)
	$(B w)
	$(B d)
)

	A string literal is either a double quoted string, a wysiwyg quoted
	string, an escape sequence, or a hex string.
	<p>

	Wysiwyg quoted strings are enclosed by r" and ".
	All characters between
	the r" and " are part of the string except for $(I EndOfLine) which is
	regarded as a single \n character.
	There are no escape sequences inside r" ":

---------------
r"hello"
r"c:\root\foo.exe"
r"ab\n"			// string is 4 characters, 'a', 'b', '\', 'n'
---------------

	An alternate form of wysiwyg strings are enclosed by backquotes,
	the ` character. The ` character is not available on some keyboards
	and the font rendering of it is sometimes indistinguishable from
	the regular ' character. Since, however, the ` is rarely used,
	it is useful to delineate strings with " in them.

---------------
`hello`
`c:\root\foo.exe`
`ab\n`			// string is 4 characters, 'a', 'b', '\', 'n'
---------------

	Double quoted strings are enclosed by "". Escape sequences can be
	embedded into them with the typical \ notation.
	$(I EndOfLine) is regarded as a single \n character.

---------------
"hello"
"c:\\root\\foo.exe"
"ab\n"			// string is 3 characters, 'a', 'b', and a linefeed
"ab
"			// string is 3 characters, 'a', 'b', and a linefeed
---------------

	Escape strings start with a \ and form an escape character sequence.
	Adjacent escape strings are concatenated:

<pre>
\n			the linefeed character
\t			the tab character
\"			the double quote character
\012			octal
\x1A			hex
\u1234			wchar character
\U00101234		dchar character
\&amp;reg;			&reg; dchar character
\r\n			carriage return, line feed
</pre>

	$(P Undefined escape sequences are errors.
	Although string literals are defined to be composed of
	UTF characters, the octal and hex escape sequences allow
	the insertion of arbitrary binary data.
	\u and \U escape sequences can only be used to insert
	valid UTF characters.
	)

	$(P Hex strings allow string literals to be created using hex data.
	The hex data need not form valid UTF characters.
	)

--------------
x"0A"			// same as "\x0A"
x"00 FBCD 32FD 0A"	// same as "\x00\xFB\xCD\x32\xFD\x0A"
--------------

	Whitespace and newlines are ignored, so the hex data can be
	easily formatted.
	The number of hex characters must be a multiple of 2.
	<p>

	Adjacent strings are concatenated with the ~ operator, or by simple
	juxtaposition:

--------------
"hello " ~ "world" ~ \n	// forms the string 'h','e','l','l','o',' ',
			// 'w','o','r','l','d',linefeed
--------------

	The following are all equivalent:

-----------------
"ab" "c"
r"ab" r"c"
r"a" "bc"
"a" ~ "b" ~ "c"
\x61"bc"
-----------------

	The optional $(I Postfix) character gives a specific type
	to the string, rather than it being inferred from the context.
	This is useful when the type cannot be unambiguously inferred,
	such as when overloading based on string type. The types corresponding
	to the postfix characters are:
	<p>

	$(TABLE1
	<tr>
	<th> Postfix
	<th> Type
	<tr>
	<td> $(B c) </td>
	<td> char[ ]
	<tr>
	<td> $(B w) </td>
	<td> wchar[ ]
	<tr>
	<td> $(B d) </td>
	<td> dchar[ ]
	)

---
"hello"c          // char[]
"hello"w          // wchar[]
"hello"d          // dchar[]
---

<h3>$(LNAME2 characterliteral, Character Literals)</h3>

$(GRAMMAR
$(I CharacterLiteral):
	$(B ') $(I SingleQuotedCharacter) $(B ')

$(I SingleQuotedCharacter)
	$(I Character)
	$(I EscapeSequence)
)

	Character literals are a single character or escape sequence
	enclosed by single quotes, ' '.

<h3>$(LNAME2 integerliteral, Integer Literals)</h3>

$(GRAMMAR
$(I IntegerLiteral):
	$(I Integer)
	$(I Integer) $(I IntegerSuffix)

$(I Integer):
	$(I Decimal)
	$(I Binary)
	$(I Octal)
	$(I Hexadecimal)
	$(I Integer)

$(I IntegerSuffix):
	$(B L)
	$(B u)
	$(B U)
	$(B Lu)
	$(B LU)
	$(B uL)
	$(B UL)

$(I Decimal):
	$(B 0)
	$(I NonZeroDigit)
	$(I NonZeroDigit) $(I DecimalDigits)

$(I Binary):
	$(B 0b) $(I BinaryDigits)
	$(B 0B) $(I BinaryDigits)

$(I Octal):
	$(B 0) $(I OctalDigits)

$(I Hexadecimal):
	$(B 0x) $(I HexDigits)
	$(B 0X) $(I HexDigits)

$(I NonZeroDigit):
	$(B 1)
	$(B 2)
	$(B 3)
	$(B 4)
	$(B 5)
	$(B 6)
	$(B 7)
	$(B 8)
	$(B 9)

$(I DecimalDigits):
	$(I DecimalDigit)
	$(I DecimalDigit) $(I DecimalDigits)

$(I DecimalDigit):
	$(B 0)
	$(I NonZeroDigit)
	$(B _)

$(I BinaryDigits):
	$(I BinaryDigit)
	$(I BinaryDigit) $(I BinaryDigits)

$(I BinaryDigit):
	$(B 0)
	$(B 1)
	$(B _)

$(I OctalDigits):
	$(I OctalDigit)
	$(I OctalDigit) $(I OctalDigits)

$(I OctalDigit):
	$(B 0)
	$(B 1)
	$(B 2)
	$(B 3)
	$(B 4)
	$(B 5)
	$(B 6)
	$(B 7)
	$(B _)

$(I HexDigits):
	$(I HexDigit)
	$(I HexDigit) $(I HexDigits)

$(I HexDigit):
	$(I DecimalDigit)
	$(B a)
	$(B b)
	$(B c)
	$(B d)
	$(B e)
	$(B f)
	$(B A)
	$(B B)
	$(B C)
	$(B D)
	$(B E)
	$(B F)
)

	Integers can be specified in decimal, binary, octal, or hexadecimal.
<p>
	Decimal integers are a sequence of decimal digits.
<p>
	Binary integers are a sequence of binary digits preceded
	by a '0b'.
<p>
	Octal integers are a sequence of octal digits preceded by a '0'.
<p>
	Hexadecimal integers are a sequence of hexadecimal digits preceded
	by a '0x' or followed by an 'h'.
<p>
	Integers can have embedded '_' characters, which are ignored.
	The embedded '_' are useful for formatting long literals, such
	as using them as a thousands separator:

-------------
123_456		// 123456
1_2_3_4_5_6_	// 123456
-------------

	Integers can be immediately followed by one 'L' or one 'u' or both.
<p>
	The type of the integer is resolved as follows:
	<p>

	$(TABLE1
	<tr>
	<th> Decimal Literal
	<th> Type
	<tr>
	<td> 0 .. 2147483647
	<td> int
	<tr>
	<td> 2147483648 .. 9223372036854775807
	<td> long
	<tr>
	<th> Decimal Literal, L Suffix
	<th> Type
	<tr>
	<td> 0L .. 9223372036854775807L
	<td> long
	<tr>
	<th> Decimal Literal, U Suffix
	<th> Type
	<tr>
	<td> 0U .. 4294967295U
	<td> uint
	<tr>
	<td> 4294967296U .. 18446744073709551615U
	<td> ulong
	<tr>
	<th> Decimal Literal, UL Suffix
	<th> Type
	<tr>
	<td> 0UL .. 18446744073709551615UL
	<td> ulong

	<tr>
	<th> Non-Decimal Literal
	<th> Type
	<tr>
	<td> 0x0 .. 0x7FFFFFFF
	<td> int
	<tr>
	<td> 0x80000000 .. 0xFFFFFFFF
	<td> uint
	<tr>
	<td> 0x100000000 .. 0x7FFFFFFFFFFFFFFF
	<td> long
	<tr>
	<td> 0x8000000000000000 .. 0xFFFFFFFFFFFFFFFF
	<td> ulong
	<tr>
	<th> Non-Decimal Literal, L Suffix
	<th> Type
	<tr>
	<td> 0x0L .. 0x7FFFFFFFFFFFFFFFL
	<td> long
	<tr>
	<td> 0x8000000000000000L .. 0xFFFFFFFFFFFFFFFFL
	<td> ulong
	<tr>
	<th> Non-Decimal Literal, U Suffix
	<th> Type
	<tr>
	<td> 0x0U .. 0xFFFFFFFFU
	<td> uint
	<tr>
	<td> 0x100000000UL .. 0xFFFFFFFFFFFFFFFFUL
	<td> ulong
	<tr>
	<th> Non-Decimal Literal, UL Suffix
	<th> Type
	<tr>
	<td> 0x0UL .. 0xFFFFFFFFFFFFFFFFUL
	<td> ulong

	)


<h3>$(LNAME2 floatliteral, Floating Literals)</h3>

$(GRAMMAR
$(I FloatLiteral):
	$(I Float)
	$(I Float) $(I FloatSuffix)
	$(I Float) $(I ImaginarySuffix)
	$(I Float) $(I FloatSuffix) $(I ImaginarySuffix)

$(I Float):
	$(I DecimalFloat)
	$(I HexFloat)

$(I DecimalFloat):
	$(I DecimalDigits) $(B .)
	$(I DecimalDigits) $(B .) $(I DecimalDigits)
	$(I DecimalDigits) $(B .) $(I DecimalDigits) $(I DecimalExponent)
	$(B .) $(I Decimal)
	$(B .) $(I Decimal) $(I DecimalExponent)
	$(I DecimalDigits) $(I DecimalExponent)

$(I DecimalExponent)
	$(B e) $(I DecimalDigits)
	$(B E) $(I DecimalDigits)
	$(B e+) $(I DecimalDigits)
	$(B E+) $(I DecimalDigits)
	$(B e-) $(I DecimalDigits)
	$(B E-) $(I DecimalDigits)

$(I HexFloat):
	$(I HexPrefix) $(I HexDigits) $(B .)
	$(I HexPrefix) $(I HexDigits) $(B .) $(I HexDigits)
	$(I HexPrefix) $(I HexDigits) $(B .) $(I HexDigits) $(I HexExponent)
	$(I HexPrefix) $(B .) $(I HexDigits)
	$(I HexPrefix) $(B .) $(I HexDigits) $(I HexExponent)
	$(I HexPrefix) $(I HexDigits) $(I HexExponent)

$(I HexPrefix):
	$(B 0x)
	$(B 0X)

$(I HexExponent)
	$(B p) $(I DecimalDigits)
	$(B P) $(I DecimalDigits)
	$(B p+) $(I DecimalDigits)
	$(B P+) $(I DecimalDigits)
	$(B p-) $(I DecimalDigits)
	$(B P-) $(I DecimalDigits)

$(I FloatSuffix):
	$(B f)
	$(B F)
	$(B L)

$(I ImaginarySuffix):
	$(B i)
)

	Floats can be in decimal or hexadecimal format,
	as in standard C.
	<p>

	Hexadecimal floats are preceded with a $(B 0x) and the
	exponent is a $(B p)
	or $(B P) followed by a decimal number serving as the exponent
	of 2.
	<p>

	Floating literals can have embedded '_' characters, which are ignored.
	The embedded '_' are useful for formatting long literals to
	make them more readable, such
	as using them as a thousands separator:

---------
123_456.567_8		// 123456.5678
1_2_3_4_5_6_._5_6_7_8	// 123456.5678
1_2_3_4_5_6_._5e-6_	// 123456.5e-6
---------

	Floating literals with no suffix are of type double.
	Floats can be followed by one $(B f), $(B F),
	or $(B L) suffix.
	The $(B f) or $(B F) suffix means it is a
	float, and $(B L) means it is a real.
	<p>

	If a floating literal is followed by $(B i), then it is an
	$(I ireal) (imaginary) type.
	<p>

	Examples:

---------
0x1.FFFFFFFFFFFFFp1023		// double.max
0x1p-52				// double.epsilon
1.175494351e-38F		// float.min
6.3i				// idouble 6.3
6.3fi				// ifloat 6.3
6.3Li				// ireal 6.3
---------

	It is an error if the literal exceeds the range of the type.
	It is not an error if the literal is rounded to fit into
	the significant digits of the type.
	<p>

	Complex literals are not tokens, but are assembled from
	real and imaginary expressions in the semantic analysis:

---------
4.5 + 6.2i		// complex number
---------

<h3>$(LNAME2 keyword, Keywords)</h3>

	Keywords are reserved identifiers.

$(GRAMMAR
$(I Keyword):
	$(B abstract)
	$(B alias)
	$(B align)
	$(B asm)
	$(B assert)
	$(B auto)

	$(B body)
	$(B bool)
	$(B break)
	$(B byte)

	$(B case)
	$(B cast)
	$(B catch)
	$(B cdouble)
	$(B cent)
	$(B cfloat)
	$(B char)
	$(B class)
	$(B const)
	$(B continue)
	$(B creal)

	$(B dchar)
	$(B debug)
	$(B default)
	$(B delegate)
	$(B delete)
	$(B deprecated)
	$(B do)
	$(B double)

	$(B else)
	$(B enum)
	$(B export)
	$(B extern)

	$(B false)
	$(B final)
	$(B finally)
	$(B float)
	$(B for)
	$(B foreach)
	$(B foreach_reverse)
	$(B function)

	$(B goto)

	$(B idouble)
	$(B if)
	$(B ifloat)
	$(B import)
	$(B in)
	$(B inout)
	$(B int)
	$(B interface)
	$(B invariant)
	$(B ireal)
	$(B is)

	$(B lazy)
	$(B long)

	$(B mixin)
	$(B module)

	$(B new)
	$(B null)

	$(B out)
	$(B override)

	$(B package)
	$(B pragma)
	$(B private)
	$(B protected)
	$(B public)

	$(B real)
	$(B return)

	$(B scope)
	$(B short)
	$(B static)
	$(B struct)
	$(B super)
	$(B switch)
	$(B synchronized)

	$(B template)
	$(B this)
	$(B throw)
	$(B true)
	$(B try)
	$(B typedef)
	$(B typeid)
	$(B typeof)

	$(B ubyte)
	$(B ucent)
	$(B uint)
	$(B ulong)
	$(B union)
	$(B unittest)
	$(B ushort)

	$(B version)
	$(B void)
	$(B volatile)

	$(B wchar)
	$(B while)
	$(B with)
)

<h3>$(LNAME2 specialtokens, Special Tokens)</h3>

	These tokens are replaced with other tokens according to the following
	table:
	<p>

	$(TABLE1
	<tr>
	<th> Special Token
	<th> Replaced with...
	<tr>
	<td> $(B __FILE__)
	<td> string literal containing source file name
	<tr>
	<td> $(B __LINE__)
	<td> integer literal of the current source line number
	<tr>
	<td> $(B __DATE__)
	<td> string literal of the date of compilation "$(I mmm dd yyyy)"
	<tr>
	<td> $(B __TIME__)
	<td> string literal of the time of compilation "$(I hh:mm:ss)"
	<tr>
	<td> $(B __TIMESTAMP__)
	<td> string literal of the date and time of compilation "$(I www mmm dd hh:mm:ss yyyy)"
	)
	<p>

<h3>$(LNAME2 specialtokenseq, Special Token Sequences)</h3>

	Special token sequences are processed by the lexical analyzer, may
	appear between any other tokens, and do not affect the syntax
	parsing.
	<p>

	There is currently only one special token sequence, $(TT #line).

$(GRAMMAR
$(I SpecialTokenSequence)
	$(B # line) $(I Integer) $(I EndOfLine)
	$(B # line) $(I Integer) $(I Filespec) $(I EndOfLine)

$(I Filespec)
	$(B ") $(I Characters) $(B ")
)

	This sets the source line number to $(I Integer),
	and optionally the source file 	name to $(I Filespec),
	beginning with the next line of source text.
	The source file and line number is used for printing error messages
	and for mapping generated code back to the source for the symbolic
	debugging output.
	<p>

	For example:

-----------------
int #line 6 "foo\bar"
x;			// this is now line 6 of file foo\bar
-----------------

	Note that the backslash character is not treated specially inside
	$(I Filespec) strings.

)

Macros:
	TITLE=D Programming Language - Lexical
	WIKI=Lex

